@@ -1,4 +1,5 @@
 use std::{
+    fmt::Display,
     path::{Path, PathBuf},
     rc::Rc,
     str::FromStr,
@@ -12,9 +13,88 @@ use url::Url;
 
 use crate::{
     error::{Error as AppError, Result},
-    item::DTItem,
+    item::Operate,
 };
 
+/// Helper type for a group's [name]
+///
+/// [name]: Group::name
+#[derive(Clone, Debug, Default, Deserialize, PartialEq)]
+pub struct GroupName(pub PathBuf);
+impl Display for GroupName {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.write_str(&self.0.to_string_lossy())
+    }
+}
+impl GroupName {
+    /// Gets the first component of this name, components are separated by
+    /// slashes.
+    pub fn main(&self) -> String {
+        let first_comp: PathBuf = self.0.components().take(1).collect();
+        first_comp.to_string_lossy().to_string()
+    }
+    /// Checks if this name is empty.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// # use dt_core::{config:: GroupName, error::Error as AppError};
+    /// assert!(GroupName("a".into()).validate().is_ok());
+    /// assert!(GroupName("a/b/c".into()).validate().is_ok());
+    /// assert!(GroupName("/starts/with/slash".into()).validate().is_err());
+    /// assert!(GroupName("relative/../path".into()).validate().is_err());
+    /// # Ok::<(), AppError>(())
+    /// ```
+    pub fn validate(&self) -> Result<()> {
+        if self
+            .0
+            .components()
+            .any(|comp| comp.as_os_str().to_string_lossy() == "..")
+        {
+            Err(AppError::ConfigError(
+                "Group name should not contain relative component".to_owned(),
+            ))
+        } else if self.0.starts_with("/") {
+            Err(AppError::ConfigError(
+                "Group name should not start with slash".to_owned(),
+            ))
+        } else if self.0 == PathBuf::from_str("").unwrap() {
+            Err(AppError::ConfigError(
+                "Group name should not be empty".to_owned(),
+            ))
+        } else {
+            Ok(())
+        }
+    }
+    /// Returns a PathBuf, which adds a [`subgroup_prefix`] to each of the
+    /// components other than the main component.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// # use std::{str::FromStr, path::PathBuf};
+    /// # use dt_core::config::GroupName;
+    /// # use pretty_assertions::assert_eq;
+    /// let gn = GroupName("gui/gtk".into());
+    /// assert_eq!(
+    ///     gn.with_subgroup_prefix("#"),
+    ///     PathBuf::from_str("gui/#gtk").unwrap(),
+    /// );
+    /// ```
+    ///
+    /// [`subgroup_prefix`]: SubgroupPrefix
+    pub fn with_subgroup_prefix(&self, subgroup_prefix: &str) -> PathBuf {
+        PathBuf::from(self.main()).join(
+            self.0
+                .iter()
+                .skip(1)
+                .map(|comp| {
+                    subgroup_prefix.to_owned() + &comp.to_string_lossy()
+                })
+                .collect::<PathBuf>(),
+        )
+    }
+}
 /// Helper type for config key [`staging`]
 ///
 /// [`staging`]: GlobalConfig::staging
@@ -29,45 +109,12 @@ impl Default for StagingPath {
         }
     }
 }
-/// Helper type for config key `allow_overwrite`
-///
-/// [`allow_overwrite`]: GlobalConfig::allow_overwrite
-#[derive(Clone, Copy, Debug, Deserialize)]
-pub struct AllowOverwrite(pub bool);
-#[allow(clippy::derivable_impls)]
-impl Default for AllowOverwrite {
-    fn default() -> Self {
-        Self(false)
-    }
-}
-/// Helper type for config key `hostname_sep`
-///
-/// [`hostname_sep`]: GlobalConfig::hostname_sep
-#[derive(Clone, Debug, Deserialize)]
-pub struct HostnameSeparator(pub String);
-impl Default for HostnameSeparator {
-    fn default() -> Self {
-        Self("@@".to_owned())
-    }
-}
-/// Helper type for config key `rename`
-///
-/// [`rename`]: GlobalConfig::rename
-#[derive(Clone, Debug, Deserialize)]
-pub struct RenamingRules(pub Vec<RenamingRule>);
-#[allow(clippy::derivable_impls)]
-impl Default for RenamingRules {
-    fn default() -> Self {
-        Self(Vec::new())
-    }
-}
 /// Syncing methods.
 #[derive(Clone, Copy, Debug, Deserialize, PartialEq)]
 pub enum SyncMethod {
     /// Instructs syncing module to directly copy each item from source to
     /// target.
     Copy,
-
     /// Instructs syncing module to first copy iach item from source to its
     /// staging directory, then symlink staged items from their staging
     /// directory to target.
@@ -78,160 +125,59 @@ impl Default for SyncMethod {
         SyncMethod::Symlink
     }
 }
-
-/// The configuration object deserialized from configuration file, every
-/// field of it is optional.
-#[derive(Clone, Debug, Default, Deserialize)]
-#[serde(default)]
-pub struct DTConfig {
-    /// Sets fallback behaviours.
-    pub global: GlobalConfig,
-
-    /// Defines values for templating.
-    pub context: ContextConfig,
-
-    /// Groups containing local files.
-    pub local: Vec<LocalGroup>,
-
-    /// Groups containing remote files.
-    pub remote: Vec<RemoteGroup>,
-}
-
-impl FromStr for DTConfig {
-    type Err = AppError;
-
-    /// Loads configuration from string.
-    fn from_str(s: &str) -> Result<Self> {
-        toml::from_str::<Self>(s)?.expand_tilde().validate()
+/// Helper type for config key [`subgroup_prefix`]
+///
+/// [`subgroup_prefix`]: GlobalConfig::subgroup_prefix
+#[derive(Clone, Debug, Deserialize)]
+pub struct SubgroupPrefix(pub String);
+impl Default for SubgroupPrefix {
+    fn default() -> Self {
+        Self("#".to_owned())
     }
 }
-
-impl DTConfig {
-    /// Loads configuration from a file.
-    pub fn from_path(path: impl AsRef<Path>) -> Result<Self> {
-        let path = path.as_ref();
-        let confstr = std::fs::read_to_string(path).unwrap_or_else(|_| {
-            panic!("Could not load config from '{}'", path.display())
-        });
-        Self::from_str(&confstr)
+/// Helper type for config key [`allow_overwrite`]
+///
+/// [`allow_overwrite`]: GlobalConfig::allow_overwrite
+#[derive(Clone, Copy, Debug, Deserialize)]
+pub struct AllowOverwrite(pub bool);
+#[allow(clippy::derivable_impls)]
+impl Default for AllowOverwrite {
+    fn default() -> Self {
+        Self(false)
     }
-
-    /// Construct another [`DTConfig`] object with only groups with given
-    /// names remaining, unmatched given names are ignored.
-    pub fn filter_names(self, group_names: Vec<String>) -> Self {
-        Self {
-            global: self.global,
-            context: self.context,
-            local: self
-                .local
-                .iter()
-                .filter(|l| group_names.iter().any(|n| l.name == *n))
-                .map(|l| l.to_owned())
-                .collect(),
-            remote: self
-                .remote
-                .iter()
-                .filter(|l| group_names.iter().any(|n| l.name == *n))
-                .map(|l| l.to_owned())
-                .collect(),
-        }
+}
+/// Helper type for config key [`ignore_failure`]
+///
+/// [`ignore_failure`]: GlobalConfig::ignore_failure
+#[derive(Clone, Copy, Debug, Deserialize)]
+pub struct IgnoreFailure(pub bool);
+#[allow(clippy::derivable_impls)]
+impl Default for IgnoreFailure {
+    fn default() -> Self {
+        Self(false)
     }
-
-    /// Validates config object.  After this, the original `global` and
-    /// `context` sections are referenced by each group via an [Rc] and can be
-    /// safely ignored in further processing.
-    ///
-    /// [Rc]: std::rc::Rc
-    fn validate(self) -> Result<Self> {
-        if !self.context.0.is_table() {
-            return Err(AppError::ConfigError(
-                "`context` is expected to be a table".to_owned(),
-            ));
-        }
-
-        let global_ref = Rc::new(self.global.to_owned());
-        let context_ref = Rc::new(self.context.to_owned());
-
-        let mut ret: Self = self;
-
-        for group in &mut ret.local {
-            group.global = Rc::clone(&global_ref);
-            group.context = Rc::clone(&context_ref);
-            group.validate()?;
-        }
-        for group in &mut ret.remote {
-            group.global = Rc::clone(&global_ref);
-            group.context = Rc::clone(&context_ref);
-            group.validate()?;
-        }
-
-        Ok(ret)
+}
+/// Helper type for config key [`hostname_sep`]
+///
+/// [`hostname_sep`]: GlobalConfig::hostname_sep
+#[derive(Clone, Debug, Deserialize)]
+pub struct HostnameSeparator(pub String);
+impl Default for HostnameSeparator {
+    fn default() -> Self {
+        Self("@@".to_owned())
     }
-
-    fn expand_tilde(self) -> Self {
-        let mut ret = self;
-
-        // Expand tilde in `global.staging`
-        let staging = &mut ret.global.staging;
-        *staging = if *staging == StagingPath("".into()) {
-            log::warn!("Empty staging path is replaced to '.'");
-            StagingPath(".".into())
-        } else {
-            StagingPath(
-                PathBuf::from_str(&shellexpand::tilde(
-                    staging.0.to_str().unwrap_or_else(|| {
-                        panic!(
-                            "Failed expanding tilde in `global.staging` ({})",
-                            staging.0.display(),
-                        )
-                    }),
-                ))
-                .unwrap(),
-            )
-        };
-
-        // Expand tilde in `base` and `target` of `local`
-        for group in &mut ret.local {
-            // `local.base`
-            group.base = if group.base == PathBuf::from_str("").unwrap() {
-                log::warn!("[{}]: Empty base is replaced to '.'", group.name);
-                ".".into()
-            } else {
-                PathBuf::from_str(&shellexpand::tilde(
-                    group.base.to_str().unwrap_or_else(|| {
-                        panic!(
-                            "Failed expanding tilde in `local.base` '{}'",
-                            group.base.display(),
-                        )
-                    }),
-                ))
-                .unwrap()
-            };
-
-            // `local.target`
-            group.target = if group.target == PathBuf::from_str("").unwrap() {
-                log::warn!(
-                    "[{}]: Empty target is replaced to '.'",
-                    group.name,
-                );
-                ".".into()
-            } else {
-                PathBuf::from_str(&shellexpand::tilde(
-                    group.target.to_str().unwrap_or_else(|| {
-                        panic!(
-                            "Failed expanding tilde in `local.target` '{}'",
-                            group.target.display(),
-                        )
-                    }),
-                ))
-                .unwrap()
-            };
-        }
-
-        ret
+}
+/// Helper type for config key [`rename`]
+///
+/// [`rename`]: GlobalConfig::rename
+#[derive(Clone, Debug, Deserialize)]
+pub struct RenamingRules(pub Vec<RenamingRule>);
+#[allow(clippy::derivable_impls)]
+impl Default for RenamingRules {
+    fn default() -> Self {
+        Self(Vec::new())
     }
 }
-
 /// Scope of a group, used to resolve _priority_ of possibly duplicated items,
 /// to ensure every target path is pointed from only one source item.
 ///
@@ -317,24 +263,168 @@ pub enum DTScope {
     /// recommended for directories that contains config files for many
     /// un-categorized applications.
     General,
-
     /// The scope for a specific app, it's priority is higher than
     /// [`General`] while lower than [`Dropin`].
     ///
     /// [`General`]: DTScope::General
     /// [`Dropin`]: DTScope::Dropin
     App,
-
     /// The scope for drop-in replacements, it has the highest priority.
     Dropin,
 }
-
 impl Default for DTScope {
     fn default() -> Self {
         DTScope::General
     }
 }
 
+/// The configuration object deserialized from configuration file, every
+/// field of it is optional.
+#[derive(Clone, Debug, Default, Deserialize)]
+#[serde(default)]
+pub struct DTConfig {
+    /// Sets fallback behaviours.
+    pub global: GlobalConfig,
+
+    /// Defines values for templating.
+    pub context: ContextConfig,
+
+    /// Groups containing local files.
+    pub local: Vec<LocalGroup>,
+
+    /// Groups containing remote files.
+    pub remote: Vec<RemoteGroup>,
+}
+
+impl FromStr for DTConfig {
+    type Err = AppError;
+
+    /// Loads configuration from string.
+    fn from_str(s: &str) -> Result<Self> {
+        toml::from_str::<Self>(s)?.expand_tilde().validate()
+    }
+}
+
+impl DTConfig {
+    /// Loads configuration from a file.
+    pub fn from_path(path: impl AsRef<Path>) -> Result<Self> {
+        let path = path.as_ref();
+        if let Ok(confstr) = std::fs::read_to_string(path) {
+            Self::from_str(&confstr)
+        } else {
+            Err(AppError::ConfigError(format!(
+                "Could not load config from '{}'",
+                path.display(),
+            )))
+        }
+    }
+
+    /// Construct another [`DTConfig`] object with groups that match given
+    /// filters.  Groups are matched hierarchically, e.g. a filter `a/b` will
+    /// select `a/b/c` and `a/b/d`, but not `a/bcd`.
+    pub fn filter_names(self, group_names: Vec<String>) -> Self {
+        Self {
+            global: self.global,
+            context: self.context,
+            local: self
+                .local
+                .iter()
+                .filter(|l| {
+                    group_names.iter().any(|n| l.name.0.starts_with(n))
+                })
+                .map(|l| l.to_owned())
+                .collect(),
+            remote: self
+                .remote
+                .iter()
+                .filter(|l| {
+                    group_names.iter().any(|n| l.name.0.starts_with(n))
+                })
+                .map(|l| l.to_owned())
+                .collect(),
+        }
+    }
+
+    /// Validates config object.  After this, the original `global` and
+    /// `context` sections are referenced by each group via an [Rc] and can be
+    /// safely ignored in further processing.
+    ///
+    /// [Rc]: std::rc::Rc
+    fn validate(self) -> Result<Self> {
+        if !self.context.0.is_table() {
+            return Err(AppError::ConfigError(
+                "`context` is expected to be a table".to_owned(),
+            ));
+        }
+
+        let global_ref = Rc::new(self.global.to_owned());
+        let context_ref = Rc::new(self.context.to_owned());
+
+        let mut ret: Self = self;
+
+        for group in &mut ret.local {
+            group.global = Rc::clone(&global_ref);
+            group.context = Rc::clone(&context_ref);
+            group.validate()?;
+        }
+        for group in &mut ret.remote {
+            group.global = Rc::clone(&global_ref);
+            group.context = Rc::clone(&context_ref);
+            group.validate()?;
+        }
+
+        Ok(ret)
+    }
+
+    fn expand_tilde(self) -> Self {
+        let mut ret = self;
+
+        // Expand tilde in `global.staging`
+        let staging = &mut ret.global.staging;
+        *staging = if *staging == StagingPath("".into()) {
+            log::warn!("Empty staging path is replaced to '.'");
+            StagingPath(".".into())
+        } else {
+            StagingPath(
+                PathBuf::from_str(&shellexpand::tilde(
+                    &staging.0.to_string_lossy(),
+                ))
+                .unwrap(),
+            )
+        };
+
+        // Expand tilde in `base` and `target` of `local`
+        for group in &mut ret.local {
+            // `local.base`
+            group.base = if group.base == PathBuf::from_str("").unwrap() {
+                log::warn!("[{}]: Empty base is replaced to '.'", group.name);
+                ".".into()
+            } else {
+                PathBuf::from_str(&shellexpand::tilde(
+                    &group.base.to_string_lossy(),
+                ))
+                .unwrap()
+            };
+
+            // `local.target`
+            group.target = if group.target == PathBuf::from_str("").unwrap() {
+                log::warn!(
+                    "[{}]: Empty target is replaced to '.'",
+                    group.name,
+                );
+                ".".into()
+            } else {
+                PathBuf::from_str(&shellexpand::tilde(
+                    &group.target.to_string_lossy(),
+                ))
+                .unwrap()
+            };
+        }
+
+        ret
+    }
+}
+
 /// A single renaming rule, used for generating names for target files which
 /// are different from their sources.
 #[derive(Clone, Debug, Deserialize_tuple)]
@@ -391,6 +481,13 @@ pub struct GlobalConfig {
     #[serde(default)]
     pub method: SyncMethod,
 
+    /// A string to be prepended to a subgroup's name when creating its
+    /// staging directory with the [`Symlink`] syncing method.
+    ///
+    /// [`Symlink`]: SyncMethod::Symlink
+    #[serde(default)]
+    pub subgroup_prefix: SubgroupPrefix,
+
     /// Whether to allow overwriting existing files.
     ///
     /// This alters syncing behaviours when the target file exists.  If set
@@ -401,6 +498,14 @@ pub struct GlobalConfig {
     #[serde(default)]
     pub allow_overwrite: AllowOverwrite,
 
+    /// Whether to treat errors omitted during syncing as warnings.  It has a
+    /// [per-group counterpart] to set per-group behaviours.  Note that errors
+    /// occured before or after syncing are NOT affected.
+    ///
+    /// [per-group counterpart]: Group::ignore_failure
+    #[serde(default)]
+    pub ignore_failure: IgnoreFailure,
+
     /// The hostname separator.
     ///
     /// Specifies default value when [`Group::hostname_sep`] is not set.
@@ -430,7 +535,10 @@ impl Default for ContextConfig {
 
 /// Configures how items are grouped.
 #[derive(Default, Clone, Deserialize, Debug)]
-pub struct Group<BaseType> {
+pub struct Group<T>
+where
+    T: Operate,
+{
     /// The global config object loaded from DT's config file.  This field
     /// _does not_ appear in the config file, but is only used by DT
     /// internally.  Skipping deserializing is achieved via serde's
@@ -450,7 +558,7 @@ pub struct Group<BaseType> {
     pub context: Rc<ContextConfig>,
 
     /// Name of this group, used as namespace in staging root directory.
-    pub name: String,
+    pub name: GroupName,
 
     /// The priority of this group, used to resolve possibly duplicated
     /// items.  See [`DTScope`] for details.
@@ -493,12 +601,12 @@ pub struct Group<BaseType> {
     /// (in this case, the directory where [DT] is being executed).
     ///
     /// [DT]: https://github.com/blurgyy/dt
-    pub base: BaseType,
+    pub base: T,
 
     /// Paths (relative to [`base`]) to the items to be synced.
     ///
     /// [`base`]: Group::base
-    pub sources: Vec<PathBuf>,
+    pub sources: Vec<T>,
 
     /// The path of the parent dir of the final synced items.
     ///
@@ -566,7 +674,8 @@ pub struct Group<BaseType> {
     /// On a machine with hostname set to `watson`, the below configuration
     /// (extraneous keys are omitted here)
     ///
-    /// ```toml [[local]]
+    /// ```toml
+    /// [[local]]
     /// ...
     /// hostname_sep = "@@"
     ///
@@ -592,11 +701,24 @@ pub struct Group<BaseType> {
     /// (regardless of this option).
     pub allow_overwrite: Option<AllowOverwrite>,
 
+    /// (Optional) Whether to treat errors omitted during syncing of this
+    /// group as warnings.  Note that errors occured before or after syncing
+    /// are NOT affected.
+    pub ignore_failure: Option<IgnoreFailure>,
+
     /// (Optional) Syncing method, overrides [`global.method`] key.
     ///
     /// [`global.method`]: GlobalConfig::method
     pub method: Option<SyncMethod>,
 
+    /// A string to be prepended to a subgroup's name when creating its
+    /// staging directory with the [`Symlink`] syncing method, overrides
+    /// [`global.subgroup_prefix`] key.
+    ///
+    /// [`Symlink`]: SyncMethod::Symlink
+    /// [`global.subgroup_prefix`]: GlobalConfig::subgroup_prefix
+    pub subgroup_prefix: Option<SubgroupPrefix>,
+
     /// (Optional) Renaming rules, appends to [`global.rename`].
     ///
     /// [`global.rename`]: GlobalConfig::rename
@@ -604,9 +726,12 @@ pub struct Group<BaseType> {
     pub rename: RenamingRules,
 }
 
-impl<BaseType> Group<BaseType> {
-    /// Gets the [`allow_overwrite`] key from a `Group` object,
-    /// falls back to the `allow_overwrite` from provided global config.
+impl<T> Group<T>
+where
+    T: Operate,
+{
+    /// Gets the [`allow_overwrite`] key from a `Group` object, falls back to
+    /// the `allow_overwrite` from its parent global config.
     ///
     /// [`allow_overwrite`]: Group::allow_overwrite
     pub fn is_overwrite_allowed(&self) -> bool {
@@ -616,8 +741,30 @@ impl<BaseType> Group<BaseType> {
         }
     }
 
-    /// Gets the [`method`] key from a `Group` object, falls back
-    /// to the `method` from provided global config.
+    /// Gets the [`ignore_failure`] key from a `Group` object, falls back to
+    /// the `ignore_failure` from its parent global config.
+    ///
+    /// [`ignore_failure`]: Group::ignore_failure
+    pub fn is_failure_ignored(&self) -> bool {
+        match self.ignore_failure {
+            Some(ignore_failure) => ignore_failure.0,
+            _ => self.global.ignore_failure.0,
+        }
+    }
+
+    /// Gets the absolute path to this group's staging directory, with the
+    /// subgroup components padded with configured [`subgroup_prefix`]es.
+    ///
+    /// [`subgroup_prefix`]: Group::subgroup_prefix
+    pub fn get_staging_dir(&self) -> PathBuf {
+        self.global
+            .staging
+            .0
+            .join(self.name.with_subgroup_prefix(&self.get_subgroup_prefix()))
+    }
+
+    /// Gets the [`method`] key from a `Group` object, falls back to the
+    /// `method` from its parent global config.
     ///
     /// [`method`]: Group::method
     pub fn get_method(&self) -> SyncMethod {
@@ -627,8 +774,19 @@ impl<BaseType> Group<BaseType> {
         }
     }
 
-    /// Gets the [`hostname_sep`] key from a `Group` object, falls
-    /// back to the [`hostname_sep`] from provided global config.
+    /// Gets the [`subgroup_prefix`] key from a `Group` object, falls back to
+    /// the `subgroup_prefix` from its parent global config.
+    ///
+    /// [`subgroup_prefix`]: Group::subgroup_prefix
+    pub fn get_subgroup_prefix(&self) -> String {
+        match &self.subgroup_prefix {
+            Some(prefix) => prefix.0.to_owned(),
+            _ => self.global.subgroup_prefix.0.to_owned(),
+        }
+    }
+
+    /// Gets the [`hostname_sep`] key from a `Group` object, falls back to the
+    /// [`hostname_sep`] from its parent global config.
     ///
     /// [`hostname_sep`]: Group::hostname_sep
     pub fn get_hostname_sep(&self) -> String {
@@ -640,12 +798,14 @@ impl<BaseType> Group<BaseType> {
 
     /// Gets the list of [renaming rules] of this group, which is an array
     /// of (REGEX, SUBSTITUTION) tuples composed of [`global.rename`] and
-    /// [`local.rename`], used in [`DTItem::make_target`] to rename the item.
+    /// [`group.rename`], used in [`Operate::make_target`] to rename the item.
+    /// The returned list is a combination of the rules from global config and
+    /// the group's own rules.
     ///
     /// [renaming rules]: Group::rename
     /// [`global.rename`]: GlobalConfig::rename
-    /// [`local.rename`]: Group::rename
-    /// [`DTItem::make_target`]: crate::item::DTItem::make_target
+    /// [`group.rename`]: Group::rename
+    /// [`Operate::make_target`]: crate::item::Operate::make_target
     pub fn get_renaming_rules(&self) -> Vec<RenamingRule> {
         let mut ret: Vec<RenamingRule> = Vec::new();
         for r in &self.global.rename.0 {
@@ -663,7 +823,7 @@ impl<BaseType> Group<BaseType> {
     /// [context]: DTConfig::context
     pub fn is_templated(&self) -> bool {
         match self.context.0.as_table() {
-            Some(map) => map.get(&self.name).is_some(),
+            Some(map) => map.get(&self.name.main()).is_some(),
             None => false,
         }
     }
@@ -671,13 +831,12 @@ impl<BaseType> Group<BaseType> {
     /// Validates this group with readonly access to the filesystem.  The
     /// following cases are denied:
     ///
-    ///   1. Empty group name
-    ///   2. Slash in group name
-    ///   3. Source item referencing parent (because items are first populated
+    ///   1. Invalid group name
+    ///   2. Source item referencing parent (because items are first populated
     ///      to the [`staging`] directory, and the structure under the
     ///      [`staging`] directory depends on their original relative path to
     ///      their [`base`])
-    ///   4. TODO: Current group contains unimplemented [`ignored`] field
+    ///   3. TODO: Current group contains unimplemented [`ignored`] field
     ///
     /// NOTE: When [`base`] is empty, sources will be looked up in the cwd of
     /// the process.
@@ -685,25 +844,16 @@ impl<BaseType> Group<BaseType> {
     /// [`ignored`]: Group::ignored
     /// [`base`]: Group::base
     fn _validate_no_fs_query(&self) -> Result<()> {
-        // 1. Empty group name
-        if self.name.is_empty() {
-            return Err(AppError::ConfigError("empty group name".to_owned()));
-        }
-        // 2. Slash in group name
-        if self.name.contains('/') {
-            return Err(AppError::ConfigError(format!(
-                "group name '{}' contains the '/' character",
-                self.name,
-            )));
-        }
-        // 3. Source item referencing parent
-        if self.sources.iter().any(|s| s.starts_with("../")) {
+        // 1. Invalid group name
+        self.name.validate()?;
+        // 2. Source item referencing parent
+        if self.sources.iter().any(|s| s.is_twisted()) {
             return Err(AppError::ConfigError(format!(
                 "source item references parent directory in group '{}'",
                 self.name,
             )));
         }
-        // 4. Current group contains unimplemented `ignored` field
+        // 3. Current group contains unimplemented `ignored` field
         if self.ignored.is_some() {
             todo!("`ignored` array works poorly and I decided to implement it in the future");
         }
@@ -737,7 +887,8 @@ impl<BaseType> Group<BaseType> {
             }
 
             // 2. Path to staging root contains readonly parent directory
-            if staging_path.parent_readonly() {
+            // NOTE: Must convert to an absolute path before checking readonly
+            if staging_path.absolute()?.is_parent_readonly() {
                 return Err(AppError::ConfigError(
                     "staging root path cannot be created due to insufficient permissions"
                         .to_owned(),
@@ -754,7 +905,8 @@ impl<BaseType> Group<BaseType> {
         }
 
         // 4. Path to target contains readonly parent directory
-        if self.target.parent_readonly() {
+        // NOTE: Must convert to an absolute path before checking readonly
+        if self.target.to_owned().absolute()?.is_parent_readonly() {
             return Err(AppError::ConfigError(format!(
                 "target path cannot be created due to insufficient permissions in group '{}'",
                 self.name,
@@ -773,19 +925,18 @@ impl LocalGroup {
     ///
     /// - Checks without querying the filesystem
     ///
-    ///   1. Empty group name
-    ///   2. Slash in group name
-    ///   3. Source item referencing parent (because items are first populated
+    ///   1. Empty [group name]
+    ///   2. Source item referencing parent (because items are first populated
     ///      to the [`staging`] directory, and the structure under the
     ///      [`staging`] directory depends on their original relative path to
     ///      their [`base`])
-    ///   4. Current group contains unimplemented [`ignored`] field
+    ///   3. Current group contains unimplemented [`ignored`] field
     ///
-    ///   5. Target and base are the same
-    ///   6. Base contains [`hostname_sep`]
-    ///   7. Source item is absolute (same reason as above)
-    ///   8. Source item contains bad globbing pattern
-    ///   9. Source item contains [`hostname_sep`]
+    ///   4. Target and base are the same
+    ///   5. Base contains [`hostname_sep`]
+    ///   6. Source item is absolute (same reason as above)
+    ///   7. Source item contains bad globbing pattern
+    ///   8. Source item contains [`hostname_sep`]
     ///
     /// - Checks that need to query the filesystem
     ///
@@ -796,6 +947,8 @@ impl LocalGroup {
     ///   3. Wrong type of existing [`target`] path
     ///   4. Path to [`target`] contains readonly parent directory
     ///
+    ///   5. Base is unreadable
+    ///
     /// [group name]: LocalGroup::name
     /// [`base`]: LocalGroup::base
     /// [`target`]: LocalGroup::target
@@ -818,7 +971,7 @@ impl LocalGroup {
 
         // 6. Base contains hostname_sep
         let hostname_sep = self.get_hostname_sep();
-        if self.base.to_str().unwrap().contains(&hostname_sep) {
+        if self.base.to_string_lossy().contains(&hostname_sep) {
             return Err(AppError::ConfigError(format!(
                 "base directory contains hostname_sep ({}) in group '{}'",
                 hostname_sep, self.name,
@@ -857,7 +1010,7 @@ impl LocalGroup {
 
         // 9. Source item contains hostname_sep
         if self.sources.iter().any(|s| {
-            let s = s.to_str().unwrap();
+            let s = s.to_string_lossy();
             s.contains(&hostname_sep)
         }) {
             return Err(AppError::ConfigError(format!(
@@ -870,6 +1023,15 @@ impl LocalGroup {
         // 1-4
         self._validate_with_fs_query()?;
 
+        // 5. Base is unreadable
+        if self.base.exists() {
+            // Check read permission of `base`
+            if let Err(e) = std::fs::read_dir(&self.base) {
+                log::error!("Could not read base '{}'", self.base.display());
+                return Err(e.into());
+            }
+        }
+
         Ok(())
     }
 }
@@ -880,18 +1042,40 @@ pub type RemoteGroup = Group<Url>;
 impl RemoteGroup {
     /// Validates this remote group, the following cases are denied:
     ///
-    ///   1. Empty group name
-    ///   2. Empty target
-    ///   3. Slash in group name
-    ///   4. Source item referencing parent (because items are first populated
+    /// - Checks without querying the filesystem
+    ///
+    ///   1. Empty [group name]
+    ///   2. Empty [`target`]
+    ///   3. Source item referencing parent (because items are first populated
     ///      to the [`staging`] directory, and the structure under the
     ///      [`staging`] directory depends on their original relative path to
     ///      their [`base`])
-    ///   5. Current group contains unimplemented [`ignored`] field
+    ///   4. Current group contains unimplemented [`ignored`] field
+    ///
+    /// - Checks that need to query the filesystem
+    ///
+    ///   1. Wrong type of existing [`staging`] path (if using the
+    ///      [`Symlink`] method)
+    ///   2. Path to staging root contains readonly parent directory (if
+    ///      using the [`Symlink`] method)
+    ///   3. Wrong type of existing [`target`] path
+    ///   4. Path to [`target`] contains readonly parent directory
+    ///
+    /// [group name]: LocalGroup::name
+    /// [`base`]: LocalGroup::base
+    /// [`target`]: LocalGroup::target
+    /// [`staging`]: GlobalConfig::staging
+    /// [`ignored`]: Group::ignored
+    /// [`Symlink`]: SyncMethod::Symlink
     fn validate(&self) -> Result<()> {
+        // - Checks without querying the filesystem --------------------------
         // 1-5
         self._validate_no_fs_query()?;
 
+        // - Checks that need to query the filesystem ------------------------
+        // 1-4
+        self._validate_with_fs_query()?;
+
         Ok(())
     }
 }
@@ -1112,6 +1296,55 @@ mod validation {
     use super::DTConfig;
     use crate::error::Error as AppError;
 
+    #[test]
+    fn relative_component_in_group_name() -> Result<(), Report> {
+        if let Err(err) = DTConfig::from_str(
+            r#"
+[[local]]
+name = "a/../b"
+base = "~"
+sources = []
+target = ".""#,
+        ) {
+            assert_eq!(
+                err,
+                AppError::ConfigError(
+                    "Group name should not contain relative component"
+                        .to_owned(),
+                ),
+                "{}",
+                err,
+            );
+            Ok(())
+        } else {
+            Err(eyre!("This config should not be loaded because a group's name contains relative component"))
+        }
+    }
+
+    #[test]
+    fn prefix_slash_in_group_name() -> Result<(), Report> {
+        if let Err(err) = DTConfig::from_str(
+            r#"
+[[local]]
+name = "/a/b/c/d"
+base = "~"
+sources = []
+target = ".""#,
+        ) {
+            assert_eq!(
+                err,
+                AppError::ConfigError(
+                    "Group name should not start with slash".to_owned(),
+                ),
+                "{}",
+                err,
+            );
+            Ok(())
+        } else {
+            Err(eyre!("This config should not be loaded because a group's name starts with a slash"))
+        }
+    }
+
     #[test]
     fn empty_group_name() -> Result<(), Report> {
         if let Err(err) = DTConfig::from_str(
@@ -1121,41 +1354,18 @@ name = ""
 base = "~"
 sources = []
 target = ".""#,
-        ) {
-            assert_eq!(
-                err,
-                AppError::ConfigError("empty group name".to_owned()),
-                "{}",
-                err,
-            );
-            Ok(())
-        } else {
-            Err(eyre!("This config should not be loaded because a group's name is empty"))
-        }
-    }
-
-    #[test]
-    fn slash_in_group_name() -> Result<(), Report> {
-        if let Err(err) = DTConfig::from_str(
-            r#"
-[[local]]
-name = "this/group/name/contains/slash"
-base = "~"
-sources = []
-target = "/tmp""#,
         ) {
             assert_eq!(
                 err,
                 AppError::ConfigError(
-                    "group name 'this/group/name/contains/slash' contains the '/' character"
-                        .to_owned()
+                    "Group name should not be empty".to_owned(),
                 ),
                 "{}",
                 err,
             );
             Ok(())
         } else {
-            Err(eyre!("This config should not be loaded because a group name contains slash"))
+            Err(eyre!("This config should not be loaded because a group's name is empty"))
         }
     }
 
@@ -1320,6 +1530,25 @@ mod validation_physical {
         get_testroot, prepare_directory, prepare_file,
     };
 
+    #[test]
+    fn non_existent_relative_staging_and_target() -> Result<(), Report> {
+        if let Err(err) = DTConfig::from_str(
+            r#"
+[global]
+staging = "staging-882b842397c5b44929b9c5f4e83130c9-dir"
+
+[[local]]
+name = "readable relative non-existent target"
+base = "base-7f2f7ff8407a330751f13dc5ec86db1b-dir"
+sources = ["b1db25c31c23950132a44f6faec2005c"]
+target = "target-ce59cb1aea35e22e43195d4a444ff2e7-dir""#,
+        ) {
+            Err(eyre!("Non-existent, relative but readable staging/target path should be loaded without error (got error :'{}')", err))
+        } else {
+            Ok(())
+        }
+    }
+
     #[test]
     fn staging_is_file() -> Result<(), Report> {
         let staging_path = prepare_file(
@@ -1528,6 +1757,65 @@ target = "{}"
             ))
         }
     }
+
+    #[test]
+    fn base_unreadable() -> Result<(), Report> {
+        let base = prepare_file(
+            get_testroot().join("base_unreadable").join("base-but-file"),
+            0o311,
+        )?;
+        if let Err(err) = DTConfig::from_str(&format!(
+            r#"
+[[local]]
+name = "base unreadable (not a directory)"
+base = "{}"
+sources = []
+target = ".""#,
+            base.display(),
+        )) {
+            assert_eq!(
+                err,
+                AppError::IoError("Not a directory (os error 20)".to_owned(),),
+                "{}",
+                err,
+            );
+        } else {
+            return Err(eyre!(
+                "This config should not be loaded because base is not a directory",
+            ));
+        }
+
+        let base = prepare_directory(
+            get_testroot()
+                .join("base_unreadable")
+                .join("base-unreadable"),
+            0o311,
+        )?;
+        if let Err(err) = DTConfig::from_str(&format!(
+            r#"
+[[local]]
+name = "base unreadable (permission denied)"
+base = "{}"
+sources = []
+target = ".""#,
+            base.display(),
+        )) {
+            assert_eq!(
+                err,
+                AppError::IoError(
+                    "Permission denied (os error 13)".to_owned(),
+                ),
+                "{}",
+                err,
+            );
+        } else {
+            return Err(eyre!(
+                "This config should not be loaded because insufficient permissions to base",
+            ));
+        }
+
+        Ok(())
+    }
 }
 
 // Author: Blurgy <gy@blurgy.xyz>
