@@ -4,13 +4,11 @@ use std::{
     rc::Rc,
 };
 
-use handlebars::Handlebars;
-
 use crate::{
     config::*,
     error::{Error as AppError, Result},
-    item::DTItem,
-    registry::DTRegistry,
+    item::Operate,
+    registry::{Register, Registry},
 };
 
 /// Expands tildes and globs in [`sources`], returns the updated config
@@ -27,7 +25,6 @@ use crate::{
 /// [`global.staging`]: crate::config::GlobalConfig::staging
 /// [`base`]: crate::config::Group::base
 /// [`target`]: crate::config::Group::target
-/// [`[[local]]`]: crate::config::Group
 fn expand(config: DTConfig) -> Result<DTConfig> {
     let mut ret = DTConfig {
         // Remove `global` and `context` in expanded configuration object.
@@ -42,32 +39,21 @@ fn expand(config: DTConfig) -> Result<DTConfig> {
     for original in config.local {
         let mut next = LocalGroup {
             global: Rc::clone(&original.global),
-            base: original.base.absolute()?,
+            base: original.base.to_owned().absolute()?,
             sources: Vec::new(),
-            target: original.target.absolute()?,
+            target: original.target.to_owned().absolute()?,
             ..original.to_owned()
         };
 
         let group_hostname_sep = original.get_hostname_sep();
 
         // Check for host-specific `base`
-        let host_specific_base = next.base.host_specific(&group_hostname_sep);
+        let host_specific_base =
+            next.base.to_owned().host_specific(&group_hostname_sep);
         if host_specific_base.exists() {
             next.base = host_specific_base;
         }
 
-        // Above process does not guarantee the `base` to exist, since a
-        // warning will be emitted later in the expanding process (see
-        // function `expand_recursive()`), just don't attempt to read
-        // non-existent `base` here by first checking its existence.
-        if next.base.exists() {
-            // Check read permission of `base`
-            if let Err(e) = std::fs::read_dir(&next.base) {
-                log::error!("Could not read base '{}'", next.base.display());
-                return Err(e.into());
-            }
-        }
-
         // Check for host-specific `sources`
         let sources: Vec<PathBuf> = original
             .sources
@@ -127,7 +113,7 @@ fn expand_recursive(
         };
 
         let initial: Vec<PathBuf> =
-            glob::glob_with(path.to_str().unwrap(), globbing_options)?
+            glob::glob_with(&path.to_string_lossy(), globbing_options)?
                 // Extract value from Result<PathBuf>
                 .map(|x| {
                     x.unwrap_or_else(|_| {
@@ -143,7 +129,8 @@ fn expand_recursive(
                 // hosts, replace current path to its host-specific
                 // counterpart if it exists.
                 .map(|x| {
-                    let host_specific_x = x.host_specific(hostname_sep);
+                    let host_specific_x =
+                        x.to_owned().host_specific(hostname_sep);
                     if host_specific_x.exists() {
                         host_specific_x
                     } else {
@@ -152,7 +139,7 @@ fn expand_recursive(
                 })
                 // Convert to absolute paths
                 .map(|x| {
-                    x.absolute().unwrap_or_else(|_| {
+                    x.to_owned().absolute().unwrap_or_else(|_| {
                         panic!(
                             "Failed converting to absolute path '{}'",
                             x.display(),
@@ -194,7 +181,8 @@ fn expand_recursive(
             // hosts, replace current path to its host-specific
             // counterpart if it exists.
             .map(|x| {
-                let host_specific_x = x.host_specific(hostname_sep);
+                let host_specific_x =
+                    x.to_owned().host_specific(hostname_sep);
                 if host_specific_x.exists() {
                     host_specific_x
                 } else {
@@ -235,7 +223,7 @@ fn resolve(config: DTConfig) -> Result<DTConfig> {
     for i in 0..config.local.len() {
         let current_priority = &config.local[i].scope;
         for s in &config.local[i].sources {
-            let t = s.make_target(
+            let t = s.to_owned().make_target(
                 &config.local[i].get_hostname_sep(),
                 &config.local[i].base,
                 &config.local[i].target,
@@ -269,8 +257,9 @@ fn resolve(config: DTConfig) -> Result<DTConfig> {
                 sources: group
                     .sources
                     .iter()
-                    .filter(|s| {
+                    .filter(|&s| {
                         let t = s
+                            .to_owned()
                             .make_target(
                                 &group.get_hostname_sep(),
                                 &group.base,
@@ -319,11 +308,8 @@ pub fn sync(config: DTConfig, dry_run: bool) -> Result<()> {
     log::trace!("Local groups to process: {:#?}", config.local);
 
     let config = expand(config)?;
-    let registry = Rc::new(
-        Handlebars::new()
-            .register_templates(&config)?
-            .register_helpers()?,
-    );
+    let registry =
+        Rc::new(Registry::default().register_helpers()?.load(&config)?);
 
     for group in &config.local {
         log::info!("Local group: [{}]", group.name);
@@ -349,10 +335,23 @@ pub fn sync(config: DTConfig, dry_run: bool) -> Result<()> {
         let group_ref = Rc::new(group.to_owned());
         for spath in &group.sources {
             if dry_run {
-                spath.populate_dry(Rc::clone(&group_ref))?;
+                if let Err(e) = spath.populate_dry(Rc::clone(&group_ref)) {
+                    if group.is_failure_ignored() {
+                        log::warn!("Error ignored: {}", e);
+                    } else {
+                        return Err(e);
+                    }
+                }
             } else {
-                spath
-                    .populate(Rc::clone(&group_ref), Rc::clone(&registry))?;
+                if let Err(e) = spath
+                    .populate(Rc::clone(&group_ref), Rc::clone(&registry))
+                {
+                    if group.is_failure_ignored() {
+                        log::warn!("Error ignored: {}", e);
+                    } else {
+                        return Err(e);
+                    }
+                }
             }
         }
     }
@@ -375,73 +374,6 @@ mod tests {
             get_testroot, prepare_directory, prepare_file,
         };
 
-        #[test]
-        fn base_unreadable() -> Result<(), Report> {
-            let base = prepare_file(
-                get_testroot().join("base_unreadable").join("base-but-file"),
-                0o311,
-            )?;
-            if let Err(err) = expand(
-                DTConfig::from_str(&format!(
-                    r#"
-[[local]]
-name = "base unreadable (not a directory)"
-base = "{}"
-sources = []
-target = ".""#,
-                    base.display(),
-                ))
-                .unwrap(),
-            ) {
-                assert_eq!(
-                    err,
-                    AppError::IoError(
-                        "Not a directory (os error 20)".to_owned(),
-                    ),
-                    "{}",
-                    err,
-                );
-            } else {
-                return Err(eyre!(
-                "This config should not be loaded because base is not a directory",
-            ));
-            }
-
-            let base = prepare_directory(
-                get_testroot()
-                    .join("base_unreadable")
-                    .join("base-unreadable"),
-                0o311,
-            )?;
-            if let Err(err) = expand(
-                DTConfig::from_str(&format!(
-                    r#"
-[[local]]
-name = "base unreadable (permission denied)"
-base = "{}"
-sources = []
-target = ".""#,
-                    base.display(),
-                ))
-                .unwrap(),
-            ) {
-                assert_eq!(
-                    err,
-                    AppError::IoError(
-                        "Permission denied (os error 13)".to_owned(),
-                    ),
-                    "{}",
-                    err,
-                );
-            } else {
-                return Err(eyre!(
-                "This config should not be loaded because insufficient permissions to base",
-            ));
-            }
-
-            Ok(())
-        }
-
         #[test]
         fn unreadable_source() -> Result<(), Report> {
             // setup
@@ -493,7 +425,7 @@ target = "{}""#,
         use color_eyre::Report;
         use pretty_assertions::assert_eq;
 
-        use crate::{config::*, item::DTItem};
+        use crate::{config::*, item::Operate};
 
         use super::super::expand;
         use crate::utils::testing::{
